#    Diagnostic and test task for the  Lyrtech VHS-ADC

set frame  VHSADC
set title "Lyrtech VHS-ADC Client"
set icon   VHSADC

set started 0

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set LogFlag 1

set verbose 0

set VHSADCServerList {localhost nndhcp176}
set VHSADCServer "select a server"
set VHSADCHost ""

set boardID -1
set maxboards -1
set boardlist "none"

set waveform "undefined"
set directory $env(HOME)

set fpgadirectory $env(HOME)
set fpgaffile "undefined"

set clkfile "undefined"
set clkdirectory $env(HOME)

set savefile "undefined"
set savedirectory $env(HOME)

proc VHSADCClient__Vhsadac16ReadFpgaReg  {args} {
    midas-warning "Please select a VHSADC WebServer"
    error "No VHSADC WebServer selected"
}

proc VHSADCClient__Vhsadac16WriteFpgaReg  {args} {
    midas-warning "Please select a VHSADC WebServer"
    error "No VHSADC WebServer selected"
}

proc VHSADCClient__Vhsadac16Sdram_Read  {args} {
    midas-warning "Please select a VHSADC WebServer"
    error "No VHSADC WebServer selected"
}

proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode]} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}

proc make-frame {} {
   global env VHSADCServerList

set rows 12
set cols 60

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :text WEBSERVERT         :label "VHSADC WebServer" :w 20  :value "select a server"
    :choice-stack WEBSERVERC :label "" :strings $VHSADCServerList
    :hskip 30
    :checkbox VERBOSE        :label "Enable Verbose Option" 

    :next-row T
    :number MODULE-NUM :label "VHS-ADC Board Selection:" :min -1 :max -1 :w 5 :value -1
    :menubutton MODULE-SEL :label "" :menu "none"
    :hskip 50
    :button INFO :label "Report Board Information"  :command click-boardinfo

  :panel P1 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P2 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20; :row-gap 10

     :hskip 40
      :checkbox TRIG :label "Digital Trigger Enabled" :deaf T 
      :checkbox ACQ  :label "Acquisition RUN State" :deaf T
      :button UPDATE :label "Update" :command update_

  :panel P3 :layout V :padx 20 :pady 5
    :layout H; :col-gap 20; :row-gap 10

    :button CLEAR   :label "Clear log window"     :command ":modify LOG :empty T"
    :button REPORT  :label "Show board registers" :command click-boardregisters 
    :button SAVE    :label "Save Settings"        :command click-save
    :button RESTORE :label "Restore Settings"     :command click-restore
    :next-row T

    :button P51 :label "FPGA Programming" :w 20 :command do_fpga_programming
    :button P52 :label "Gain Control"     :w 20 :command do_gain_control
    :button P53 :label "User Registers"   :w 20 :command do_user_registers

    :next-row T
    :button P54 :label "Clock settings"   :w 20 :command do_clock
    :button P55 :label "ADC status"       :w 20 :command do_adc
    :button P56 :label "FPGA Acquisition" :w 20 :command do_fpgaacq

    :next-row T
    :button P57 :label "SDRAM"            :w 20 :command do_sdram
    :button P58 :label "Load Default Setup" :w 20 :command do_defaultsetup
    :button P59 :label "Extras"           :w 20 :command do_extras

   open-frame

  :galley_size LOG $rows $cols

   :show T
}

proc do_fpga_programming {} {
    upvar #0 fpgaffile filename

  :frame "[self frame-name].fpga" :label "FPGA programming" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   FPGAFILE    :label "Bitstream file for FPGA" :w 60 :value $filename
      :button FPGABROWSE :label "Browse"

      :next-row T
      :col-gap 20
      :button FPGADONE  :label "Done" :command "frame-quit [self frame-name].fpga"
      :button FPGAUPLOAD    :label "Upload FPGA bitstream file"  :command do_UpLoadfpga
      :button FPGAPGM       :label "Program FPGA"                :command do_Programfpga
      :button FPGAUPLOADPGM :label "Upload & Program FPGA"       :command do_UpLoadProgramfpga
      :button FPGARESET :label "FPGA Internal Reset" :command click-fpgareset


  :show T
}

proc do_gain_control {} {

  :frame "[self frame-name].gain" :label "Programmable gain configuration" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

     :choice-stack GAINCTRL :label "Programmable Gain Control" :strings [list register internal]
     :next-row T

      for {set i 1} {$i <= 16} {incr i} {
          :number GAINVL$i :label Ch$i :min 0 :value 4095 :valuex 6c :w 4
          :checkbox GAINEN$i :label Enable
          :next-row T
      }

      :checkbox GAINALL :label "All gains as Ch1"
      :next-row T
      :button GAINPGM   :label "Set programmable gains"

      :next-row T
      :button GAINDONE  :label "Done" :command "frame-quit [self frame-name].gain"
      :button GAINUPDTE :label "Update" :command update_gain

  :show T

   update_gain
}

proc do_user_registers {} {

  :frame "[self frame-name].user" :label "User defined control registers" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      for {set i 0} {$i <= 11} {incr i} {
          :text USERVL$i :label Ch$i :valuex 6c :w 10
          :next-row T
      }

      :button USERDONE  :label "Done" :command "frame-quit [self frame-name].user"
      :button USERUPDTE :label "Update" :command update_user

  :show T

   update_user
}

proc do_extras {} {
   global waveform

  :frame "[self frame-name].extra" :label "User defined control registers" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

    :text   WAVEFORM  :label "Waveform file" :w 40  :value $waveform
    :button BROWSEWAVEFORM :label "Browse"
    :button SHOWWAVEFORM   :label "Display"

    :next-row T

      :button EXTRASDONE  :label "Done" :command "frame-quit [self frame-name].extra"
      :button EXTARSUPDTE :label "Update" :command update_extras

  :show T

   update_extras
}

proc do_clock {} {

  :frame "[self frame-name].clock" :label "Clock Settings" :resize F :transient T

  :panel P :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

      :choice-stack CLKSRC :label "Clock Source:" :strings [list "Programmable (FS6377)" External]
      :hskip 25
      :text DCM :label "DCMs" :w 8
      :hskip 25
      :button RESETDCM :label "Reset FPGA DCM" :command ResetDCM

      :next-row T
      :choice-stack CDPCLK :label "CD PCLK"  :strings [list "PLLC/D(1) and MUXC/D(1)" "PLLC/D(2) and MUXC/D(2)"]
      :choice-stack OEPCLK :label "OE PCLK"  :strings [list Disabled Enabled]
      :choice-stack PDPCLK :label "PCLK O/P" :strings [list "Normal" "Power Down"]

      :next-row T
      :message CLKM1 :label "Programmable Clock Value Registers" :bold T
      :next-row T
      for {set i 1} {$i <= 16} {incr i} {
          :number CLKVL$i :label $i :min 0 :value 0xff :valuex 6c :w 6
          if {[expr ($i/4)*4] == $i} {:next-row T}
      }

      :button CLKPGM    :label "Program FS6377 from register values"

      :next-row T
      :text   CLKFILE   :label "Dat file for FS6377 Programmable Clock" :w 32 :valuex 32c
      :button CLKBROWSE :label "Browse"
      :button CLKLOAD   :label "Load"

    :next-row T

      :button CLKDONE  :label "Done" :command "frame-quit [self frame-name].clock"
      :button CLKUPDTE :label "Update" :command update_clock

  :show T

   update_clock
}

proc do_adc {} {

  :frame "[self frame-name].adc" :label "ADC Settings" :resize F :transient T

  :panel P :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :message ADCM1 :label "ADC Out Of Range Status" :bold T
     :next-row T

      for {set i 1} {$i <= 16} {incr i} {
          :checkbox ADC$i :label Ch$i :valuex 6c 
          if {[expr ($i/4)*4] == $i} {:next-row T}
      }

    :next-row T

      :button ADCDONE  :label "Done" :command "frame-quit [self frame-name].adc"
      :button ADCUPDTE :label "Update" :command update_adc

  :show T

   update_adc
}

set sdramaddr  0
set sdramwords 128

set sdrambase 0x0000
set sdrambasechoice [list "SDRAM - 0x0000"]

proc do_sdram {} {
   global sdrambase sdramaddr sdramwords sdrambasechoice

  :frame [self frame-name].sdram :label "Display Memory" :resize F :transient T

    :panel SDRAM :layout T :padx 10 :pady 10
     :layout H; :col-gap 10; :row-gap 10
     :choice-stack SDRAMBASEC  :label "Area base address" :strings $sdrambasechoice :value 0 :valuex 24c
     :text SDRAMBASET :label "" :value $sdrambase :w 12
     :next-row T
     :text SDRAMADDR    :label "Address Offset"  :value $sdramaddr   :valuex 24c  :w 12
     :button SDRAMADDRZ :label "Reset to zero"
     :next-row T
     :layout V
     :text SDRAMWORDS   :label "Words to read"   :value $sdramwords  :valuex 24c  :w 12
       :message SDRAMM1 :label "Memory Contents" :bold T
       :galley SDRAMINFO :rows 20 :cols 110 :deaf T :font fixed

  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

       :button SDRAMDONE :label "Done" :command "frame-quit [self frame-name].sdram"
       :button SDRAMREAD :label "Read Memory"  :command  do_sdramdisplay
       :button SDRAMCLR  :label "Clear Window" :command ":modify SDRAMINFO :append F"
  
   :show T
}

proc do_fpgaacq {} {
    global fpgaacq

  :frame "[self frame-name].fpgaacq" :label "FPGA Acquisition" :resize F :transient T

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

      :message FPGAACQP1 :label "Acquisition Options" :bold T
      :next-row T

      :choice-stack FPGAACQSF  :label "Storage Format" :strings [list Single Double Quad]
      :choice-stack FPGAACQSID :label "Storage ID"     :strings [list "Ext SDRAM"]
      :choice-stack FPGAACQSRC :label "Source ID"      :strings [list "Internal    " VHS-ADC]

      :next-row T
      :number FPGAACQFRAMESIZE :label "Frame Size" :min 32 :max 8192 :valuex 12c :w 6

  :panel P2 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

      :message FPGAACQP2 :label "Trigger Control" :bold T
      :next-row T

      :choice-stack FPGAACQTRIGSEL :label "Select" \
          :strings [list Ch0 Ch1 Ch2 Ch3 Ch4 Ch5 Ch6 Ch7 Ch8 Ch9 Ch10 Ch11 Ch12 Ch13 Ch14 Ch15] :valuex 8c
      :choice-stack FPGAACQTRIGPOL :label "Polarity" :strings [list Falling Rising] :valuex 8c
      :choice-stack FPGAACQTRIGSRC   :label "Source" :strings [list Manual Signal External] :valuex 8c
      :next-row T
      :choice-stack FPGAACQTRIGMDE   :label "Mode" :strings [list "One Shot   " Continuous] :valuex 8c
      :number FPGAACQTRIGLVL   :label "Level" :min 0 :max 65536 :w 5 :valuex 6c
      :number FPGAACQTRIGDLY   :label "Delay" :min 0 :max 65536 :w 5 :valuex 6c


  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

      :message FPGAACQP3 :label "Channel Select" :bold T
      :next-row T

      for {set i 1} {$i <= 16} {incr i} {
          :checkbox FPGAACQCH$i :label $i :valuex 2c
          if {$i == 8}  {:button FPGAACQALL  :label All  :w 5; :next-row T}
          if {$i == 16} {:button FPGAACQNONE :label None :w 5}
      }


  :panel P4 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20; :row-gap 10

      :message FPGAACQP4 :label "Acquisition Status" :bold T
      :next-row T

      :choice-stack FPGAACQTRIGRUN   :label "Trigger State"  :strings [list Stopped Running]
      :choice-stack FPGAACQTRIGSTATE :label "Acquisition State" \
           :strings [list Idle Armed Waiting Storing Ready Pre-ready "" "" "Data select error" \
                "Trigger select error" "Storage overrun error" "Packing overrun error" "Data overrun error" ]
      :hskip 75
      :button FPGAAUTO :label "Auto Update"  :command fpga_auto_update

      :next-row T
      :text FPGAACQFRAMECNT    :label "Frame Count" :w 6
      :text FPGAACQSAMPLES     :label "Samples" :w 2
      :checkbox FPGAACQFERR :label "Data Framing Error"
      :checkbox FPGAACQACT  :label "Data Available"

  :panel P5 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

      :button FPGAACQDONE  :label "Done" :command "frame-quit [self frame-name].fpgaacq"
      :button FPGAACQUPDTE :label "Update" :command update_fpgaacq
      :button FPGAACQREGS  :label "Show FPGA Acquisition Registers" :command click-fpgaregisters

  :show T

   update_fpgaacq
}

proc do_defaultsetup {args} {    ;#  load a setup compatible with our data acquisition program
    global boardlist boardID

    do_fpgaacq
    set save $boardID

    foreach boardID $boardlist {

        click-fpgareset
        click-fpgaacq SF 1 Double       ;# data format - 2 samples per word
        click-fpgaacq SRC 1 VHS-ADC
        click-fpgaacq FRAMESIZE 512 0   ;# Sample length - 512 items
        click-fpgaacq CH1 1 1           ;# Select channel 1
        click-fpgaacq TRIGSRC 2 0       ;# Trigger Source - External 
        click-fpgaacq TRIGMDE 0 0       ;# Trigger Mode - One shot
    }

    set boardID $save     ;# restore 
}

proc do_acqGO {opt} {   ;#  for external control;  opt=0  STOP;   opt!=0  GO
    global boardlist

    if {$opt == 0} then {set vl 0} else {set vl 9}

    foreach boardID $boardlist {
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x1c $vl ;# acq/trig control register
    }
}

proc do_acqARM {opt} {   ;#  for external control;  opt=0  STOP;   opt!=0  GO
    global boardlist

    if {$opt == 0} then {set vl 0} else {set vl 1}

    foreach boardID $boardlist {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x10] \
                [expr ($rc & 0x0018ffff) | ($vl << 17)]   ;# fpga trig control
   }
}

proc get_framesize {vl} {
    return [VHSADCClient__Vhsadac16ReadFpgaReg $vl [expr 0x1000 + 0x08]]
}

proc redisplay {} {
    catch {update_}
    catch {update_gain}
    catch {update_user}
    catch {update_extras}
    catch {update_clock}
    catch {update_adc}
    catch {update_fpgaacq}
}

proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n; update idletasks}


proc start args {
   global started

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
}

proc click arg {
   global verbose VHSADCHost

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer

   switch -glob $w  {

      WEBSERVERT    {click-server $v}
      WEBSERVERC    {click-server $i}

      MODULE-SEL    -
      MODULE-NUM    {click-select-module $v}

      GAINVL*       {click-gainvl [string range $w 6 end] $v}
      GAINEN*       {click-gainen [string range $w 6 end] $v}

      USERVL*       {click-uservl [string range $w 6 end] $v}

      CLKVL*        {click-clkvl [string range $w 5 end] $v}

      FPGAACQ*      {click-fpgaacq [string range $w 7 end] $v $i}

      VERBOSE       {set verbose $v; if {$VHSADCHost != ""} then {VHSADCClient__VHSADCVerbose $verbose}}
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }

    redisplay 
}

proc click-server {v} {
    global VHSADCServerList VHSADCServer
    global boardID maxboards boardlist
    global verbose started

    global VHSADCHost VHSADCPort

    set VHSADCServer $v
    :modify WEBSERVERT :value $VHSADCServer
    :modify WEBSERVERC :value [lsearch $VHSADCServerList $VHSADCServer]

    set VHSADCHost $VHSADCServer
    package require VHSADCClient

    VHSADCClient__VHSADCVerbose $verbose
    set maxboards [VHSADCClient__VHSADCInit]
    
    set boardlist {}
    for {set i 0} {$i < $maxboards} {incr i} {lappend boardlist $i}
    set boardID [lindex $boardlist 0]
    :modify MODULE-SEL :menu $boardlist
    :modify MODULE-NUM :min 0 :max [expr $maxboards-1]
    :modify MODULE-NUM :value $boardID

    insert "Initialised VHSADC Web Service at $VHSADCServer: $maxboards VHS-ADC boards discovered"

    set started 2
}

proc click-select-module {vl} {
    global boardID maxboards

    set boardID $vl
    if {$boardID < 0} {set boardID 0}
    if {$boardID >= $maxboards} {set boardID [expr $maxboards-1]}

    :modify MODULE-NUM :value $boardID
}

proc click-save {} {
    global env boardlist
    upvar #0 savedirectory  directory
    upvar #0 savefile filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Data Files} {*.dat} }
	{ {All Files } {*} }
	}

	set f [tk_getSaveFile -parent [self frame-name] \
		  -title "Save File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      if {[file extension $f] == ""} then {append f .dat}

	set filename $f   ;#  remember current choice
      set directory [file dirname $f]

      set z [catch {open $filename w} f]
      if {$z != 0} then {
          midas-warning "Cannot create/open file $filename\n$f"
          return
      }

      foreach ID $boardlist {

          foreach offset [list 0x8 0xc 0x10 0x14 0x18 0x24 0x28 0x2c 0x30 \
              0x34 0x38 0x3c 0x40 0x44 0x70 0x74 0x78 0x7c \
              0x100 0x104 0x108 0x10c 0x110 0x114 0x118 0x11c \
              0x1000 0x1004 0x1008 0x100c 0x1010 0x1014 0x1018] {

              set rc [VHSADCClient__Vhsadac16ReadFpgaReg $ID $offset]
              puts $f "$ID [format 0x%03x $offset] [format 0x%08x $rc]"
          }
      }

      close $f

     set-footer "Settings saved to $filename"
}

proc click-restore {} {
    global env boardlist boardID
    upvar #0 savedirectory  directory
    upvar #0 savefile filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Data Files} {*.dat} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "Save File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set filename $f   ;#  remember current choice
      set directory [file dirname $f]

      set z [catch {open $filename r} f]
      if {$z != 0} then {
          midas-warning "Cannot create/open file $filename\n$f"
          return
      }

      set settings [split [read $f] \n]

      close $f

      foreach setting $settings {

          if {$setting == ""} {continue}
          set ID [first $setting]
          set offset [second $setting]
          set vl [third $setting]

set vl [expr $vl & 0x7fffffff]  ;# I have a problem with the MSB

          insert "setting board $ID register [format "0x%x" $offset] to [format "0x%x" $vl]"
          set rc [VHSADCClient__Vhsadac16WriteFpgaReg $ID $offset $vl]
      }

      set saveIt $boardID
      foreach boardID $boardlist {
          click-clkpgm
          fpgaacq_channelselect             ;# to setup selected channels
      }
      set boardID $saveIt

      redisplay

     set-footer "Settings restored from $filename"
}

proc click-sdrambasec {v} {
   global  sdrambase sdrambasechoice

   set sdrambase  [last [lindex $sdrambasechoice $v]]
   :modify SDRAMBASET :value [format "0x%05x" $sdrambase]
}

proc click-sdrambaset {v} {
   global  sdrambase
   if {[scan-int $v] == $v && $v >= 0} then {set sdrambase $v}
   :modify SDRAMBASET :value [format "0x%05x" $sdrambase]
   :modify SDRAMBASEC :value -1
}

proc click-sdramaddr {v} {
   global sdramaddr
   if {[scan-int $v] == $v && $v >= 0 && $v < 0x10000} then {set sdramaddr $v}
   :modify SDRAMADDR :value [format "0x%04x" $sdramaddr]
}

proc click-sdramaddrz {args} {click-sdramaddr 0}

proc click-sdramwords {v} {
   global sdramwords
   if {[scan-int $v] == $v && $v >= 0 && $v <= 0x400} then {set sdramwords $v}
   :modify SDRAM WORDS :value [format "0x%02x" $sdramwords]
}

proc  do_sdramdisplay {} {
   global boardID
   global  sdrambase sdramaddr sdramwords
   global DataRAM

   set-footer [self frame-name].sdram "Reading data from SDRAM"

   set baseaddress [expr $sdrambase + $sdramaddr]
   set length [expr $sdramwords*4]
   set DataRAM [VHSADCClient__Vhsadac16Sdram_Read $boardID $baseaddress $length]

   set info ""
   set i 0
   set offset 0
   foreach word $DataRAM {
      if {$i == 0} {append info "[format "0x%08x" [expr $baseaddress + $offset]]: "}
      append info "[format 0x%08x $word]  "
      incr i
      if {$i == 8} then {append info "\n"; set i 0}
      incr offset 4
   }
   :modify sdramINFO :strings $info\n

   clear-footer [self frame-name].sdram

   click-sdramaddr [expr $sdramaddr + ($sdramwords * 4)]
}

proc click-fpgareset {args} {
    global boardID

    set ctrl [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x00]  ;#  board control register
    set ctrl [expr $ctrl & 0x000005f8]    ;# remove reserved bits
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x00 [expr $ctrl | 0x100] ;# set INT RST bit  (8)

    after 1000 update_
}

proc click-fpgafile {args} {
    upvar #0 fpgadirectory  directory
    upvar #0 fpgaffile filename

    set filename [first $args]
    set directory [file dirname $filename]
}

proc click-fpgabrowse {args} {
    global env
    upvar #0 fpgadirectory  directory
    upvar #0 fpgaffile filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.bit} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

	set filename $f   ;#  remember current choice
      set directory [file dirname $f]

      :modify FPGAFILE :value $filename
}

proc do_UpLoadfpga {} {
    upvar #0 fpgaffile file

    set-footer "[self frame-name].fpga"  "Uploading FPGA bitstream file.... This takes about 10 secs"
    set z [catch {VHSADCClient__VHSADCUpLoad $file} rc]
    clear-footer "[self frame-name].fpga" 

    if {$z == 0} {
         set-footer "[self frame-name].fpga"   "Uploaded file $file successful"
     } else {
         bgerror "Uploading file $file failed\n$rc"
    }
}

proc do_Programfpga {} {
    global boardID
    upvar #0 fpgaffile filename

    set-footer "[self frame-name].fpga"  "loading FPGA.... This takes about 10 secs"
    set z [catch {VHSADCClient__Vhsadac16FpgaLoad $boardID $filename} rc]
    clear-footer "[self frame-name].fpga" 

    if {$z == 0} {
        if {[string range $rc 0 3] == "Fpga"} then {
             set-footer "[self frame-name].fpga"   "FpgaLoad for board $boardID file $filename successful"
        } else {
              bgerror "FpgaLoad for board $boardID file $filename failed\n$rc"
        }
    } else {
         bgerror "FpgaLoad for board $boardID file $filename failed\n$rc"
    }
}

proc do_UpLoadProgramfpga {}  {do_UpLoadfpga; do_Programfpga}

proc click-clkfile {args} {
    upvar #0 clkdirectory  directory
    upvar #0 clkfile filename

    set filename [first $args]
    set directory [file dirname $filename]
}

proc click-clkbrowse {args} {
    global env
    upvar #0 clkdirectory  directory
    upvar #0 clkfile filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.dat} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FS6377 data File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

	set filename $f   ;#  remember current choice
      set directory [file dirname $f]

      :modify CLKFILE :value $filename
}

proc click-clkload {args} {
    global boardID
    upvar #0 clkdirectory  directory
    upvar #0 clkfile filename

      set z [catch {open $filename r} f]
      if {$z != 0} then {
          midas-warning "Cannot open file $filename\n$f"
          return
      }

      set clkdat [split [read $f] \n]

      close $f

      for {set i 0} {$i < [expr [llength $clkdat]-2]} {incr i 4} {
          set dat [expr ([lindex $clkdat [expr $i+3]] << 24) + \
                        ([lindex $clkdat [expr $i+2]] << 16) + \
                        ([lindex $clkdat [expr $i+1]] << 8) + [lindex $clkdat $i]]
          set rc [VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x0c + $i] $dat]
      }

      update_clock
}

proc click-boardregisters {args}  {
    global boardID

    insert "VHS-ADC control registers"

    set info ""
    set i 0
    foreach offset [list 0x0 0x8 0xc 0x10 0x14 0x18 0x1c 0x20 0x24 0x28 0x2c 0x30 \
       0x34 0x38 0x3c 0x40 0x44 0x70 0x74 0x78 0x7c 0x100 0x104 0x108 0x10c 0x110 0x114 0x118 0x11c] {

        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $offset]
        append info "[format 0x%03x $offset]=[format 0x%08x $rc]   "

        incr i
        if {$i == 4} {append info \n; set i 0}
    }
    insert $info
}

proc click-fpgaregisters {args}  {
    global boardID

    insert "FPGA Acquisition Registers"

    set info ""
    set i 0
    foreach offset [list 0x0 0x4 0x8 0xc 0x10 0x14 0x18 0x1c] {

        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + $offset]]
        append info "[format 0x%03x $offset]=[format 0x%08x $rc]   "

        incr i
        if {$i == 4} {append info \n; set i 0}
    }
    insert $info
}

proc ResetDCM {args} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x08 [expr ($rc & 0x39) | 0x80]  ;# set Reset DCM bit

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    if {[expr ($rc >> 7) & 1] == 1} {midas-warning "FPGA DCM reset bit NOT cleared"}

    update_clock
}

proc click-cdpclk {vl} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x08 [expr ($rc & 0x19) | ($vl << 5)]  ;# set CD PCLK bit

    update_clock
}

proc click-oepclk {vl} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x08 [expr ($rc & 0x29) | ($vl << 4)]  ;# set OEPCLK bit

    update_clock
}

proc click-pdpclk {vl} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x08 [expr ($rc & 0x31) | ($vl << 3)]  ;# set Pdown PCLK bit

    update_clock
}

proc click-clksrc {vl} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x08 [expr ($rc & 0x38) | $vl]  ;# set CLK SRC bit

    update_clock
}

proc click-trig {vl} {
    global boardID

    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x1c $vl ;# acq/trig control register
}

proc update_gain {} {
    global boardID

    set ctrl [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x24]  ;# gain configuration register

    :modify GAINCTRL :value [expr ($ctrl >> 18) & 1]
    :modify GAINALL  :value [expr ($ctrl >> 16) & 1]

    set j 1
    for {set i 0} {$i < 8} {incr i} {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x28 + ($i*4)]]  ;# gain register
        set gain [expr $rc & 0x0fff]
        set en [expr ($ctrl >> ($j-1)) & 1]
        :modify GAINVL$j :value $gain
        :modify GAINEN$j :value $en
        incr j
        set gain [expr ($rc >> 16) & 0xfff]
        set en [expr ($ctrl >> ($j-1)) & 1]
        :modify GAINVL$j :value $gain
        :modify GAINEN$j :value $en
        incr j
    }

    if {[expr ($ctrl >> 17) & 1] != 0} {insert "Warning: Program Gain bit is set"}

}

proc click-gainpgm {args} {
    global boardID

    set ctrl [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x24]  ;# gain configuration register
    set ctrl [expr $ctrl & 0x0005ffff]    ;# remove reserved bits
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x24 [expr $ctrl | 0x20000] ;# set PrG lvl bit  (17)

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x24]  ;# gain configuration  register
    if {[expr ($rc >> 17) & 1] == 1} {midas-warning "Gain programming bit NOT cleared"}

    update_gain
}

proc click-gainctrl {vl} {
    global boardID

    set ctrl [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x24]  ;# gain configuration register
    set ctrl [expr $ctrl & 0x0005ffff]    ;# remove reserved bits
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x24 [expr $ctrl | ($vl << 18)] ;# set PrG Ctrl bit  (18)

    update_gain
}

proc click-gainall {vl} {
    global boardID

    set ctrl [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x24]  ;# gain configuration register
    set ctrl [expr $ctrl & 0x0005ffff]    ;# remove reserved bits
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x24 [expr $ctrl | ($vl << 16)] ;# set Ch1 bit  (16)

    update_gain
}

proc click-gainen {ch vl} {
    global boardID

    set ctrl [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x24]  ;# gain configuration register
    set ctrl [expr $ctrl & 0x0005ffff]    ;# remove reserved bits
    set new [format 0x%x [expr ($ctrl & ~(1 << ($ch-1))) | $vl << ($ch-1)]] 
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x24 $new       ;# set Ch enable bit

    update_gain
}

proc click-gainvl {ch vl} {
    global boardID

    set i [expr ($ch-1)/2]
    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x28 + ($i*4)]]  ;# gain register

    if {[expr $ch & 1] == 1} {
        set new [format 0x%x [expr ($rc & 0x0fff0000) + $vl]]
    } else {
        set new [format 0x%x [expr ($rc & 0x0000ffff) + ($vl << 16)]]
    }

    VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x28 + ($i*4)]] $new    ;# update gain register

    update_gain
}

proc update_user {} {
    global boardID

    for {set i 0} {$i <= 3} {incr i} {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x70 + ($i*4)]]  ;# user register
        :modify USERVL$i :value [format 0x%08x $rc]
    }

    for {set i 4} {$i <= 11} {incr i} {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x100 + (($i-4)*4)]]  ;# user register
        :modify USERVL$i :value [format 0x%08x $rc]
    }
}

proc click-uservl {ch vl} {
    global boardID

   if {$ch <= 3} {
       VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x70 + ($ch*4)]]  $vl
   } else {
       VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x100 + (($ch-4)*4)]]  $vl
   }

   update_user
}

proc click-clkvl {ch vl} {
    global boardID

   set i [expr (($ch-1)/4)*4]
   set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x0c + $i]] 

   switch [expr $ch-1-((($ch-1)/4)*4)] {
       0  {set vl [expr ($rc & 0xffffff00) | $vl]}
       1  {set vl [expr ($rc & 0xffff00ff) | ($vl << 8)]}
       2  {set vl [expr ($rc & 0xff00ffff) | ($vl << 16)]}
       3  {set vl [expr ($rc & 0x00ffffff) | ($vl << 24)]}
   }

   set rc [VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x0c + $i] $vl]

   update_clock
}

proc click-clkpgm {args} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x0c]  ;# programmable clock value registers 1-4
    if {$rc == 0} {return}   ;#  no values set

    set-footer "Programming PLL CLK"

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID 0x08 [expr ($rc & 0x39) | 0x40]  ;# set PCLK bit

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register
    if {[expr ($rc >> 6) & 1] == 1} {midas-warning "PLL CLK programming bit NOT cleared"}

    ResetDCM    ;#  also calls update_clock

    clear-footer
}

proc update_clock {} {
    global boardID

    set z [catch {:query_Mapped [self frame-name].clock} rc]
    if {$z != 0 || $rc != 1} {return}

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x08]  ;# clocks control register

    :modify CLKSRC :value [expr $rc & 1]  ;# CLK SRC
    :modify CDPCLK :value [expr ($rc >> 5) & 1]
    :modify OEPCLK :value [expr ($rc >> 4) & 1]
    :modify PDPCLK :value [expr ($rc >> 3) & 1]

    if {[expr $rc & 0x100] == 0} {:modify DCM :value "NOT locked"} else {:modify DCM :value "Locked"}

    if {[expr $rc & 0x80] != 0} {insert "Warning: Reset FPGA DCM bit is set"}
    if {[expr $rc & 0x40] != 0} {insert "Warning: Program PLL CLK bit is set"}

    set j 1
    for {set i 0} {$i < 4} {incr i} {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x0c + ($i*4)]]  ;# user register
        for {set k 0} {$k < 4} {incr k} {
            :modify CLKVL$j :value [format 0x%02x [expr ($rc >> (($j-1)*8)) & 255]]
            incr j
        }
    }
}

proc update_adc {} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x20] 

    for {set i 1} {$i <= 16} {incr i} {
       :modify ADC$i :value [expr ($rc >> ($i-1)) & 1]
    }
}

proc update_fpgaacq {} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x00]]

    set StorageFormat [expr ($rc >> 24) & 255]
    if {$StorageFormat == 1} {:modify FPGAACQSF :value 0}
    if {$StorageFormat == 2} {:modify FPGAACQSF :value 1}
    if {$StorageFormat == 4} {:modify FPGAACQSF :value 2}

    set SourceID [expr ($rc >> 8) & 255]
    if {$SourceID == 16} {:modify FPGAACQSRC :value 0}
    if {$SourceID == 6} {:modify FPGAACQSRC :value 1}

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]

    :modify FPGAACQTRIGSEL :value [expr $rc & 255]
    :modify FPGAACQTRIGSRC :value [expr ($rc >> 19) & 3]
    :modify FPGAACQTRIGPOL :value [expr ($rc >> 16) & 1]
    :modify FPGAACQTRIGRUN :value [expr ($rc >> 17) & 1]
    :modify FPGAACQTRIGMDE :value [expr ($rc >> 18) & 1]

    :modify FPGAACQTRIGSTATE :value [expr ($rc >> 24) & 15]

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x14]]
    :modify FPGAACQTRIGLVL :value $rc

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x18]]
    :modify FPGAACQTRIGDLY :value $rc

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x08]]
    :modify FPGAACQFRAMESIZE :value $rc

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x0c]]
    :modify FPGAACQFRAMECNT :value [expr $rc]

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x04]]
    for {set i 1} {$i <= 16} {incr i} {
          :modify FPGAACQCH$i :value [expr ($rc >> ($i-1)) & 1]
    }

    :modify FPGAACQFERR :value [expr ($rc >> 31) & 1]
    :modify FPGAACQACT :value [expr ($rc >> 30) & 1]
    :modify FPGAACQSAMPLES :value [expr ($rc >> 16) & 31]

}

proc fpga_auto_update {args} {
    global boardID

    if {![:query_Mapped [self frame-name].fpgaacq]} {return}

    set-busy

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]

    :modify FPGAACQTRIGRUN :value [expr ($rc >> 17) & 1]
    :modify FPGAACQTRIGSTATE :value [expr ($rc >> 24) & 15]
    
    clear-busy

   after 1000 fpga_auto_update
}
    

proc click-fpgaacq {w vl i} {
    global boardID

    switch -glob $w {

       SF  {
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x00]]
           if {$i == "Single"} {set opt 1}
           if {$i == "Double"} {set opt 2}
           if {$i == "Quad"} {set opt 4}
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x00] \
                [expr ($rc & 0x00ffff00) | ($opt << 24)]
       }

       SRC  {
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x00]]
           if {$i == "Internal    "} {set opt 16}
           if {$i == "VHS-ADC"} {set opt 6}
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x00] \
                [expr ($rc & 0xffff0000) | ($opt << 8)]
       }

       FRAMESIZE  {
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x08] $vl
       }

       TRIGLVL  {
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x14] $vl
       }

       TRIGDLY  {
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x18] $vl
       }

       TRIGSEL  {
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x10] \
                [expr ($rc & 0x00ff0000) | $vl]
       }

       TRIGPOL  {
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x10] \
                [expr ($rc & 0x001e00ff) | ($vl << 16)]
       }

       TRIGMDE  {
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x10] \
                [expr ($rc & 0x001b00ff) | ($vl << 18)]
       }

       TRIGSRC  {
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x10]]
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x10] \
                [expr ($rc & 0x000700ff) | ($vl << 19)]
       }

       CH*  {
           set i [expr [string range $w 2 end] - 1]
           set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 + 0x04]]
           set mask [expr ~(1 << $i)]
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x04] \
                [expr ($rc & 0x0000ffff) & $mask | ($vl << $i)]
           fpgaacq_channelselect
       }

       ALL  {
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x04] 0x0000ffff
           fpgaacq_channelselect
       }

       NONE  {
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x1000 + 0x04] 0x00000000
           fpgaacq_channelselect
       }

    }

    update_fpgaacq
}

proc fpgaacq_channelselect {} {
    global boardID

    set register [expr 0x1000 + 0x04]   ;# data status|data control|data select register

    set-busy

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $register]
    VHSADCClient__Vhsadac16WriteFpgaReg $boardID $register 0x00800000    ;# clear storage indexes

    for {set i 0} {$i < 16} {incr i} {
       if {[expr ($rc >> $i) & 1] == 1} {
           set vl [expr ($rc & 0x0000ffff) | ($i << 16)]
           VHSADCClient__Vhsadac16WriteFpgaReg $boardID $register $vl
       }
    }

    clear-busy
}

proc update_ {} {
    global boardID

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0x1c]   ;# acquisition and trigger control

    :modify ACQ  :value [expr $rc & 1]
    :modify TRIG :value [expr ($rc >> 3) & 1]
}

proc click-boardinfo {args} {
    global boardID

    :modify LOG :empty T      ;#  empty LOG window  

#    Get FPGA Revision

    set rc1 [expr [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0] >> 16 & 255]
#    insert "VHSADCGetFPGARevision returned $rc1"

#    Get FPGA Board Status and Control Information

    set rc2 [VHSADCClient__Vhsadac16ReadFpgaReg $boardID 0]
#    insert "Vhsadac16_ReadFpgaReg VHSVIRTEX_STATUS_CTRL returned $rc2"

#    Get CPLD Registers Information

    set rc3 [VHSADCClient__Vhsadac16GetCpldRegs $boardID]
#    insert "Vhsadac16_Vhsadac16GetCpldRegs returned $rc3"

    set virtex_pgm [lindex $rc3 0]
    set adc_speed_status [lindex $rc3 1]
    set cpld_status [lindex $rc3 2]
    set cpld_revision [lindex $rc3 3]
    set pcb_revision [lindex $rc3 4]
    set sm_board_id [lindex $rc3 5]

#    print board info

    switch $sm_board_id {
         0x82  {set infos "VHS-ADC 8 - Virtex II 3000"}
         0x83  {set infos "VHS-ADC 8 - Virtex II 4000"}
         0x84  {set infos "VHS-ADC 8 - Virtex II 6000"}
         0x85  {set infos "VHS-ADC 8 - Virtex II 8000"}
         0x8a  {set infos "VHS-ADC 8 - XC4LX40"}
         0x8b  {set infos "VHS-ADC 8 - XC4LX60"}
         0x8c  {set infos "VHS-ADC 8 - XC4LX80"}
         0x8d  {set infos "VHS-ADC 8 - XC4LX100"}
         0x8e  {set infos "VHS-ADC 8 - XC4LX160"}
         0x8f  {set infos "VHS-ADC 8 - XC4SX55"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: Main Board:\t\t\t $infos"

    set uifpga_daughter_present [expr ($rc2 & 0x00000200) >> 9]
    set uidaughter_id [expr ($rc2 & 0x00000004) >> 2]
    set uidaughter_present [expr ($rc2 & 0x00000002) >> 1]

    if {$uifpga_daughter_present == 0} {
        if {$uidaughter_present == 1} {
            if {$uidaughter_id == 0} {    ;# daughter is a DAC
                set infos "VHS-DAC 8"
            } else {
                set infos "VHS-ADC 8"
            }
        } else {
            set infos "No daughter board present"
        }
    } else {
        set infos "FPGA Daughter Board"
    }

    insert "Slot\[$boardID\]: Daughter Board:\t\t $infos"
    insert "Slot\[$boardID\]: FPGA Core Revision:\t\t $rc1"

#    Get SDRAM Info

    set rc4 [VHSADCClient__Vhsadac16Sdram_AutoInit $boardID]
#    insert "Vhsadac16_Vhsadac16Sdram_AutoInit returned $rc4"

    set rc4 [VHSADCClient__Vhsadac16Sdram_DualPageMode $boardID]
#    insert "Vhsadac16_Vhsadac16Sdram_DualPageMode returned $rc4"

    set uisdram_frequency [VHSADCClient__Vhsadac16Sdram_Frequency $boardID]
#    insert "Vhsadac16_Vhsadac16Sdram_Frequency returned $uisdram_frequency"

    if {$uisdram_frequency != 0xdeadbeef} {
        set uisdram_size [VHSADCClient__Vhsadac16Sdram_Size $boardID]
#        insert "Vhsadac16_Vhsadac16Sdram_Size returned $uisdram_size"
    } else {
        set uisdram_frequency =1
    }

#    print SDRAM frequency and size

    if {$uisdram_frequency != -1}  {    ;#  if SDRAM is initialised
        insert "Slot\[$boardID\]: SDRAM Frequency:\t\t [expr $uisdram_frequency/1000000.0] MHz"
        insert "Slot\[$boardID\]: SDRAM Size:\t\t\t [expr (4.0 * $uisdram_size)/(1024*1024)] MB"
    } else {
        insert "Slot\[$boardID\]: SDRAM Not initialised"
    }

#    Get FPGA recording registers

     for {set i 0} {$i<8} {incr i} {
         set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1000 | ($i << 2)]]
         set uiacq_registers($i) $rc
     }

#    extract info from FPGA recording registers

     set uiacq_version [expr $uiacq_registers(0) & 0xff]
     set uiacq_source [expr ($uiacq_registers(0) & 0x0000ff00) >> 8]
     set uiacq_packing [expr ($uiacq_registers(0) & 0xff000000) >> 24]
    
#    Get FPGA playing registers

     for {set i 0} {$i<8} {incr i} {
         set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x1020 | ($i << 2)]]
         set uiplay_registers($i) $rc
     }

#    print SDRAM data packing

    switch $uiacq_packing {
         1  {set infos "1 Sample/32 bit Word"}
         2  {set infos "2 Samples/32 bit Word"}
         4  {set infos "4 Samples/32 bit Word"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: SDRAM Data Packing:\t\t $infos"

#    print ADC max speed

    switch [expr $adc_speed_status & 3] {
         0  {set infos "65 MHz"}
         1  {set infos "80 MHz"}
         2  {set infos "105 MHz"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: ADC MAximum Speed:\t\t $infos"

#    print ADC analog bandwidth

    switch [expr ($cpld_status & 0x18) >> 3] {
         0  {set infos "A - BW = 30MHz,DR = 26dB"}
         1  {set infos "B - BW = 100MHz,DR = 17.5dB"}
         2  {set infos "C - BW = 200MHz,DR = 14dB"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: Programmable Gain Option:\t $infos"

#    print GPIO Option info 

    switch [expr ($cpld_status & 0x4) >> 2] {
         0  {set infos "Option A - Standards I/Os (32/32)"}
         1  {set infos "Option B - LVDS pairs (12/32) + Standards I/Os (20/32)"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: GPIO Option:\t\t\t $infos"

#    print CPLD version

    switch [expr $cpld_revision] {
         0  {set infos "Revision A"}
         1  {set infos "Revision B"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: CPLD Revision:\t\t $infos"

#    print PCB revision

    switch [expr $pcb_revision] {
         0  {set infos "Revision A"}
         1  {set infos "Revision B"}
         2  {set infos "Revision C"}
         3  {set infos "Revision D"}
         default  {set infos "Unknown"}
    }

    insert "Slot\[$boardID\]: PCB Hardware Revision:\t $infos"


}

#    remaining procedures are to display the waveform written (.bin file) by the Lyrtech utility

proc update_extras {args} {}

proc click-waveform {args} {
    global waveform directory

    set  waveform [first $args]
    set directory [file dirname $file]
}

proc click-browsewaveform {args} {
    global waveform
    global env  directory

    	option add *Canvas.background  #c0c020
	set types {
	{ {Waveform Files} {*.bin} }
	{ {All Files } {*} }
	}

	set file [tk_getOpenFile -parent [self frame-name] \
		  -title "Waveform File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$file == ""} then {return}

	set waveform $file   ;#  remember current choice
        set directory [file dirname $file]

       :modify WAVEFORM :value $waveform
}

proc click-showwaveform {args} {
    global waveform

    insert "reading $waveform"

    set z [ catch {open $waveform r} f ]
    if { $z != 0 } then {midas-warning "Unable to open file $waveform\n$f"; return}

    fconfigure $f -translation binary -encoding binary

    set data [read $f]

    close $f

    set z [catch {binary scan $data s* r} m]

    midas-check-for-temp

    set file [file join [pwd] VHS.txt]

    insert "creating $file"

    set z [ catch {open $file w} f ]
    if { $z != 0 } then {midas-warning "Unable to open file $file\n$f"; return}

    foreach item $r {puts $f $item}

    close $f

    set destn [file join [pwd] VHS[date +%Y%h%d-%H.%M.%S]]

    insert "converting $file to $destn"

    set z [catch {exec ascii-to-eg -from $file -to $destn -format y} m]

    if { $z != 0 } then {midas-warning "Error converting file $file\n$m"; return}

    insert "showing $destn"

    send sd show $destn

}
